package tests;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import abstractions.Balance;
import abstractions.BalanceSpecification;


public abstract class JUnit_BalanceSpecification {
	
	BalanceSpecification balanceUnderTest;
	
	@Before
	public abstract void setUp() throws Exception;
	
	@After
	public void tearDown() throws Exception {
		balanceUnderTest = null;
	}

	/**
	 * Tests to make sure that after construction all weights are on the table and none are in the pans.<br>
	 * Also checks that the initial state is safe
	 */
	@Test 
	public void testConstructor(){
		
		Assert.assertEquals(0, balanceUnderTest.weighing());
		Assert.assertTrue(balanceUnderTest.invariant());
	}
	
	
	/**
	 * Tests {@link BalanceSpecification#weigh} and {@link Balance#maxThatCanBeWeighed()} and {@link Balance#weighing}
	 */	
	@Test 
	public void testWeighMaximum(){
		
		int maxToWeigh = balanceUnderTest.maxThatCanBeWeighed();
		balanceUnderTest.weigh(maxToWeigh);
		Assert.assertEquals(maxToWeigh, balanceUnderTest.weighing());
		Assert.assertTrue(balanceUnderTest.invariant());
	}
	
	/**
	 * Tests {@link BalanceSpecification#moveAllToTable()}
	 */
	@Test 
	public void testMoveAllToTable(){
		
		int maxToWeigh = balanceUnderTest.maxThatCanBeWeighed();
		balanceUnderTest.weigh(maxToWeigh);
		balanceUnderTest.moveAllToTable();
		Assert.assertEquals(0, balanceUnderTest.weighing());
		Assert.assertTrue(balanceUnderTest.invariant());
	}
	
	/**
	 * Tests {@link BalanceSpecification#maxNumberOfWeights}
	 */
	@Test 
	public void testMaxNumberOfWeights(){
		
		int max = balanceUnderTest.maxNumberOfWeights();
		int maxToWeigh = balanceUnderTest.maxThatCanBeWeighed();
		balanceUnderTest.weigh(maxToWeigh);
		Assert.assertEquals(max, balanceUnderTest.maxNumberOfWeights());
		balanceUnderTest.moveAllToTable();
		Assert.assertEquals(max, balanceUnderTest.maxNumberOfWeights());
		Assert.assertTrue(balanceUnderTest.invariant());
	}
	
	/**
	 * Tests {@link BalanceSpecification#getWeight}
	 */
	@Test
	public void testGetWeight(){
		int temp_weight=0;
		int sum=0;
		
		for (int i=0;  i < balanceUnderTest.maxNumberOfWeights(); i++){
		temp_weight = balanceUnderTest.getWeight(i);
        sum = sum+temp_weight;
		balanceUnderTest.addLeft(temp_weight);
		
		Assert.assertEquals(temp_weight, balanceUnderTest.getWeight(i));
		balanceUnderTest.removeLeft(temp_weight);
		Assert.assertEquals(temp_weight, balanceUnderTest.getWeight(i));
		balanceUnderTest.addRight(temp_weight);
		Assert.assertEquals(temp_weight, balanceUnderTest.getWeight(i));
		balanceUnderTest.removeRight(temp_weight);
		}
		Assert.assertEquals(balanceUnderTest.maxThatCanBeWeighed(), sum);
		Assert.assertTrue(balanceUnderTest.invariant());
	}

	/**
	 * Pick a random weight that can be weighed by the system and check that it is weighed correctly.
	 * Then repeat 100 times
	 */
    @Test
	public void testSomeRandomWeights (){
   
      final int NUMBER_OF_TESTS =100; 
      for (int count= 0; count < NUMBER_OF_TESTS; count++){
    	   balanceUnderTest.randomize();
    	   int target = balanceUnderTest.weighing();
    	   if (target <0) target = target*(-1);
    	   balanceUnderTest.weigh(target);
    	   Assert.assertEquals(target, balanceUnderTest.weighing());
      }

	}
    
    /**
     * Run {@link JUnit_BalanceSpecification#testSomeRandomWeights} on the system containing 
     * the maximum number of weights
     */
    @Test
	public abstract void testSomeRandomWeightsOnLargestSet ();
	
}