package tests;

import models.SequenceBalance;

import org.junit.Assert;
import org.junit.Test;

/**
 * Unit tests for {@link SequenceBalance} which initialises the size of the weight 
 * set to {@link  JUnit_BinaryBalance#SIZE_OF_WEIGHT_SET}
 * @author gibson
 * @version 1
 */
public class JUnit_SequenceBalance extends JUnit_BalanceSpecification{
	
	/**
     * the size of the largest sequence weight set that does not cause integer arithmetic overflow<br>
     * it is 2 to the power of 16.
     */
    final int MAXIMUM_WEIGHT = 65535; 
    
    
	static final int SIZE_OF_WEIGHT_SET = 15;
	
	public void setUp() throws Exception {

		balanceUnderTest = new SequenceBalance(SIZE_OF_WEIGHT_SET);
	}
	
	
	/**
	 * A sequence balance should be able to weigh all values between 1 and the maximum
	 */
    @Test
	public void testAllWeights (){
    	
    	int max = balanceUnderTest.maxThatCanBeWeighed();
    	
    	for (int i=1;i<=max;i++){
    		
    		balanceUnderTest.weigh(i);
    		Assert.assertEquals(i, balanceUnderTest.weighing());
    	}
	}
    
    @Test(expected=IllegalArgumentException.class)
    public void testOverflowOnMaximumWeight(){  	
    	new SequenceBalance (MAXIMUM_WEIGHT+1);
    }
    
   
    @Test(expected=IllegalArgumentException.class)
    public void testNonNegativeWeightSet(){  	
    	new SequenceBalance (-1);
    }
    
    @Override
	public void testSomeRandomWeightsOnLargestSet (){
		
		balanceUnderTest = new SequenceBalance(SequenceBalance.MAXIMUM_NUMBER_OF_SEQUENCE_WEIGHTS/2);
		super.testSomeRandomWeights();
	}


}

