package tests;

import tools.DateHeader;
import models.TernaryBalance;

/**
 * Validation test for {@link TernaryBalance}<br>
 * The test is run for a default target value of 100. <br>
 * This is to represent the balance in the state:
<pre> 
   \81,27,1/           \100, 9/
    -------             ------
       |                  |
       --------------------
                /\
 </pre>                 
 * (This value can be over-written by a valid integer parameter on the command line.) <br>
 * For the default value the expected (time-stamped) output is:            
<pre>
The target weight is set, by default, to 100.
You can override this default value by passing an integer value as a main argument parameter, if you so wish.

********************************************************************
Execution Date/Time 2014/02/21 17:12:01
*********************************************************************

 **** Testing the biggest possible weight set ****

Left : 
Right : 
Table :  1 3 9 27 81 243 729 2187 6561 19683 59049 177147 531441 1594323 4782969 14348907 43046721 129140163 387420489 1162261467
Weighing 0
balance.maxNumberOfWeights() =  20
balance.maxThatCanBeWeighed() =  1743392200

 **** Testing weighing of 100  ****

Left :  1 27 81
Right :  9
Table :  3 243 729 2187 6561 19683 59049 177147 531441 1594323 4782969 14348907 43046721 129140163 387420489 1162261467
Weighing 100

 **** Testing weighing of maximum that can be weighed  ****

Left :  1 3 9 27 81 243 729 2187 6561 19683 59049 177147 531441 1594323 4782969 14348907 43046721 129140163 387420489 1162261467
Right : 
Table : 
Weighing 1743392200

 **** Testing randomisation of balance ****

Left :  3 9 243 2187 177147 531441 4782969 1162261467
Right :  81 6561 59049 1594323 14348907 43046721 129140163
Table :  1 27 729 19683 387420489
Weighing 979559661

 **** Testing the smallest possible weight set ****

Left : 
Right : 
Table :  1
Weighing 0
balance.maxNumberOfWeights() =  1
balance.maxThatCanBeWeighed() =  1

 **** Testing weighing of 1  ****

Left :  1
Right : 
Table : 

 **** Testing weighing of 100  ****

Left :  1
Right : 
Table : 

 **** Testing exceptions in balance construction ****

 Correctly caught the overflow case: 
java.lang.IllegalArgumentException: Negative weights are not permitted

 Correctly caught the negative weights exception: 
java.lang.IllegalArgumentException: There must be at least one weight in the system
</pre>
 * @author gibson
 * @version 1
 */
public class TestTernaryBalance {
	
	
public static void main (String[] args){
		
	// The default target sum to weigh with the balance
		final int DEFAULT_TARGET = 100;
		
	    int target = DEFAULT_TARGET;
	    		
	    if (args.length ==0) {
			System.out.println("The target weight is set, by default, to "+target+
	      			           ".\nYou can override this default value by passing an integer value as "
	      			           + "a main argument parameter, if you so wish.");  	
	    }
	    
	    else try{target = Integer.parseInt(args[0]);
	             if (target<1){
	            	 System.out.println("The target weight must be greater than 0, "
	            	 		+ "so we default the value to "+ DEFAULT_TARGET+".\n");
	            	 target = DEFAULT_TARGET;}
	             else System.out.println("The target weight has been set to "+target);
	    }
	         catch (NumberFormatException exc){
	        	   System.out.println("First argument was not a valid integer");
	               System.out.println("The target weight is specifed by default to "+target);
	         }
	         
	    
		System.out.println(DateHeader.dateString());    

	    
			TernaryBalance balance = new TernaryBalance (TernaryBalance.MAXIMUM_NUMBER_OF_TERNARY_WEIGHTS);
			
			System.out.println("\n **** Testing the biggest possible weight set ****");
			System.out.println(balance);
			System.out.println("Weighing "+balance.weighing());
			System.out.println("balance.maxNumberOfWeights() =  "+balance.maxNumberOfWeights());
			System.out.println("balance.maxThatCanBeWeighed() =  "+balance.maxThatCanBeWeighed());
			
			// We can weigh any value by just putting weights in left balance, when target is in right
						
			System.out.println("\n **** Testing weighing of "+ target+"  ****");
			balance.weigh(target);
			System.out.println(balance);
			System.out.println("Weighing "+balance.weighing());
			
			System.out.println("\n **** Testing weighing of maximum that can be weighed  ****");
		    balance.weigh(balance.maxThatCanBeWeighed());
		    System.out.println(balance);
		    System.out.println("Weighing "+balance.weighing());

			System.out.println("\n **** Testing randomisation of balance ****");
			balance.randomize();System.out.println(balance);
			System.out.println("Weighing "+balance.weighing());
			
			System.out.println("\n **** Testing the smallest possible weight set ****");
			
			/**
		     * the size of the smallest binary weight set allowed
		     */
			final int MIN_NUMBER_OF_WEIGHTS = 1;
			
	        balance = new TernaryBalance (MIN_NUMBER_OF_WEIGHTS);
	        
			System.out.println(balance);
			System.out.println("Weighing "+balance.weighing());
			System.out.println("balance.maxNumberOfWeights() =  "+balance.maxNumberOfWeights());
			System.out.println("balance.maxThatCanBeWeighed() =  "+balance.maxThatCanBeWeighed());
			
			target = balance.maxThatCanBeWeighed();
			balance.weigh(target);
			System.out.println("\n **** Testing weighing of "+ target+"  ****");
			System.out.println(balance);

			target = DEFAULT_TARGET;
			balance.weigh(target);
			System.out.println("\n **** Testing weighing of "+ target+"  ****");
			System.out.println(balance);
			
			System.out.println("\n **** Testing exceptions in balance construction ****");
			
			 try {balance = new TernaryBalance (TernaryBalance.MAXIMUM_NUMBER_OF_TERNARY_WEIGHTS+1);}
			 catch (IllegalArgumentException exception )
			       {System.out.println("\n Correctly caught the overflow case: \n" +exception);}
			 
		     try {balance = new TernaryBalance (-1);}
			 catch (IllegalArgumentException exception )
				   {System.out.println("\n Correctly caught the negative weights exception: \n"+exception);
				   }
		     

		  }
	}
